from typing import Callable

import pytest
from eth_tester.exceptions import TransactionFailed
from web3 import Web3
from web3.contract import Contract
from web3.exceptions import ValidationError

from raiden_contracts.constants import (
    EMPTY_ADDRESS,
    EVENT_TOKEN_NETWORK_CREATED,
    TEST_SETTLE_TIMEOUT_MAX,
    TEST_SETTLE_TIMEOUT_MIN,
)
from raiden_contracts.tests.utils import call_and_transact
from raiden_contracts.tests.utils.constants import DEPLOYER_ADDRESS, NOT_ADDRESS
from raiden_contracts.utils.events import check_token_network_created


@pytest.mark.usefixtures("no_token_network")
def test_constructor_call(
    web3: Web3,
    get_token_network_registry: Callable,
    secret_registry_contract: Contract,
    get_accounts: Callable,
) -> None:
    """Try to create a TokenNetworkRegistry with various wrong arguments."""
    A = get_accounts(1)[0]
    chain_id = web3.eth.chain_id
    settle_min = TEST_SETTLE_TIMEOUT_MIN
    settle_max = TEST_SETTLE_TIMEOUT_MAX

    # failure with no arguments
    with pytest.raises(TypeError):
        get_token_network_registry()

    # failure with an int instead of the SecretRegistry's address
    with pytest.raises(TypeError):
        get_token_network_registry(
            _secret_registry_address=3,
            _chain_id=chain_id,
            _settlement_timeout_min=settle_min,
            _settlement_timeout_max=settle_max,
            _max_token_networks=1,
        )

    # failure with zero instead of the SecretRegistry's address
    with pytest.raises(TypeError):
        get_token_network_registry(
            _secret_registry_address=0,
            _chain_id=chain_id,
            _settlement_timeout_min=settle_min,
            _settlement_timeout_max=settle_max,
            _max_token_networks=1,
        )

    # failure with the empty string instead of the SecretRegistry's address
    with pytest.raises(TypeError):
        get_token_network_registry(
            _secret_registry_address="",
            _chain_id=chain_id,
            _settlement_timeout_min=settle_min,
            _settlement_timeout_max=settle_max,
            _max_token_networks=1,
        )

    # failure with an odd-length hex string instead of the SecretRegistry's address
    with pytest.raises(TypeError):
        get_token_network_registry(
            _secret_registry_address=NOT_ADDRESS,
            _chain_id=chain_id,
            _settlement_timeout_min=settle_min,
            _settlement_timeout_max=settle_max,
            _max_token_networks=1,
        )

    # failure with the empty string instead of a chain ID
    with pytest.raises(TypeError):
        get_token_network_registry(
            _secret_registry_address=secret_registry_contract.address,
            _chain_id="",
            _settlement_timeout_min=settle_min,
            _settlement_timeout_max=settle_max,
            _max_token_networks=1,
        )

    # failure with a string instead of a chain ID
    with pytest.raises(TypeError):
        get_token_network_registry(
            _secret_registry_address=secret_registry_contract.address,
            _chain_id="1",
            _settlement_timeout_min=settle_min,
            _settlement_timeout_max=settle_max,
            _max_token_networks=1,
        )

    # failure with a negative integer instead of a chain ID
    with pytest.raises(TypeError):
        get_token_network_registry(
            _secret_registry_address=secret_registry_contract.address,
            _chain_id=-3,
            _settlement_timeout_min=settle_min,
            _settlement_timeout_max=settle_max,
            _max_token_networks=1,
        )

    # failure with chain ID zero
    with pytest.raises(TransactionFailed, match="TNR: invalid chain id"):
        get_token_network_registry(
            _secret_registry_address=secret_registry_contract.address,
            _chain_id=0,
            _settlement_timeout_min=settle_min,
            _settlement_timeout_max=settle_max,
            _max_token_networks=1,
        )

    # failure with strings instead of the minimal challenge period
    with pytest.raises(TypeError):
        get_token_network_registry(
            [secret_registry_contract.address, chain_id, "", settle_max, 1, 1]
        )
    with pytest.raises(TypeError):
        get_token_network_registry(
            [secret_registry_contract.address, chain_id, "1", settle_max, 1, 1]
        )
    with pytest.raises(TypeError):
        get_token_network_registry(
            [secret_registry_contract.address, chain_id, -3, settle_max, 1, 1]
        )

    # failure with strings instead of the max challenge period
    with pytest.raises(TypeError):
        get_token_network_registry(
            _secret_registry_address=secret_registry_contract.address,
            _chain_id=chain_id,
            _settlement_timeout_min=settle_min,
            _settlement_timeout_max="",
            _max_token_networks=1,
        )
    with pytest.raises(TypeError):
        get_token_network_registry(
            _secret_registry_address=secret_registry_contract.address,
            _chain_id=chain_id,
            _settlement_timeout_min="settle_min,1",
            _max_token_networks=1,
        )
    with pytest.raises(TypeError):
        get_token_network_registry(
            _secret_registry_address=secret_registry_contract.address,
            _chain_id=chain_id,
            _settlement_timeout_min=settle_min,
            _settlement_timeout_max=-3,
            _max_token_networks=1,
        )

    # failure with Ethereum accounts that doesn't look like a SecretRegistry
    with pytest.raises(TransactionFailed, match="TNR: invalid SR address"):
        get_token_network_registry(
            _secret_registry_address=EMPTY_ADDRESS,
            _chain_id=chain_id,
            _settlement_timeout_min=settle_min,
            _settlement_timeout_max=settle_max,
            _max_token_networks=1,
        )
    with pytest.raises(TransactionFailed, match="TNR: invalid SR"):
        get_token_network_registry(
            _secret_registry_address=A,
            _chain_id=chain_id,
            _settlement_timeout_min=settle_min,
            _settlement_timeout_max=settle_max,
            _max_token_networks=1,
        )

    # failures with chain_id zero
    with pytest.raises(TransactionFailed, match="TNR: invalid chain id"):
        get_token_network_registry(
            _secret_registry_address=secret_registry_contract.address,
            _chain_id=0,
            _settlement_timeout_min=0,
            _settlement_timeout_max=settle_max,
            _max_token_networks=1,
        )
    with pytest.raises(TransactionFailed, match="TNR: invalid chain id"):
        get_token_network_registry(
            _secret_registry_address=secret_registry_contract.address,
            _chain_id=0,
            _settlement_timeout_min=settle_min,
            _settlement_timeout_max=0,
            _max_token_networks=1,
        )
    with pytest.raises(TransactionFailed, match="TNR: invalid chain id"):
        get_token_network_registry(
            _secret_registry_address=secret_registry_contract.address,
            _chain_id=0,
            _settlement_timeout_min=settle_max,
            _settlement_timeout_max=settle_min,
            _max_token_networks=1,
        )

    # failures with nonsense challenge periods
    with pytest.raises(TransactionFailed, match="TNR: invalid settle timeout min"):
        get_token_network_registry(
            _secret_registry_address=secret_registry_contract.address,
            _chain_id=chain_id,
            _settlement_timeout_min=0,
            _settlement_timeout_max=settle_max,
            _max_token_networks=1,
        )
    with pytest.raises(TransactionFailed, match="TNR: invalid settle timeout max"):
        get_token_network_registry(
            _secret_registry_address=secret_registry_contract.address,
            _chain_id=chain_id,
            _settlement_timeout_min=settle_min,
            _settlement_timeout_max=0,
            _max_token_networks=1,
        )
    with pytest.raises(TransactionFailed, match="TNR: invalid settle timeouts"):
        get_token_network_registry(
            _secret_registry_address=secret_registry_contract.address,
            _chain_id=chain_id,
            _settlement_timeout_min=settle_max,
            _settlement_timeout_max=settle_min,
            _max_token_networks=1,
        )

    # failures with nonsense token number limits
    with pytest.raises(TransactionFailed, match="TNR: invalid TN limit"):
        get_token_network_registry(
            _secret_registry_address=secret_registry_contract.address,
            _chain_id=chain_id,
            _settlement_timeout_min=settle_min,
            _settlement_timeout_max=settle_max,
            _max_token_networks=0,
        )
    with pytest.raises(TypeError):
        get_token_network_registry(
            _secret_registry_address=secret_registry_contract.address,
            _chain_id=chain_id,
            _settlement_timeout_min=0,
            _settlement_timeout_max=settle_max,
            _max_token_networks="limit",
        )

    get_token_network_registry(
        _secret_registry_address=secret_registry_contract.address,
        _chain_id=chain_id,
        _settlement_timeout_min=settle_min,
        _settlement_timeout_max=settle_max,
        _max_token_networks=1,
    )


@pytest.mark.usefixtures("no_token_network")
def test_constructor_call_state(
    web3: Web3, get_token_network_registry: Callable, secret_registry_contract: Contract
) -> None:
    """The constructor should set the parameters into the storage of the contract"""

    chain_id = web3.eth.chain_id

    registry = get_token_network_registry(
        _secret_registry_address=secret_registry_contract.address,
        _chain_id=chain_id,
        _settlement_timeout_min=TEST_SETTLE_TIMEOUT_MIN,
        _settlement_timeout_max=TEST_SETTLE_TIMEOUT_MAX,
        _max_token_networks=30,
    )
    assert secret_registry_contract.address == registry.functions.secret_registry_address().call()
    assert chain_id == registry.functions.chain_id().call()
    assert TEST_SETTLE_TIMEOUT_MIN == registry.functions.settlement_timeout_min().call()
    assert TEST_SETTLE_TIMEOUT_MAX == registry.functions.settlement_timeout_max().call()
    assert 30 == registry.functions.max_token_networks().call()


@pytest.mark.usefixtures("no_token_network")
def test_create_erc20_token_network_call(
    token_network_registry_contract: Contract,
    custom_token: Contract,
    get_accounts: Callable,
    channel_participant_deposit_limit: int,
    token_network_deposit_limit: int,
) -> None:
    """Calling createERC20TokenNetwork() with various wrong arguments"""

    A = get_accounts(1)[0]
    fake_token_contract = token_network_registry_contract.address

    # failure with no arguments
    with pytest.raises(ValidationError):
        token_network_registry_contract.functions.createERC20TokenNetwork()

    # failures with integers instead of a Token contract address
    with pytest.raises(ValidationError):
        token_network_registry_contract.functions.createERC20TokenNetwork(
            3, channel_participant_deposit_limit, token_network_deposit_limit
        )
    with pytest.raises(ValidationError):
        token_network_registry_contract.functions.createERC20TokenNetwork(
            0, channel_participant_deposit_limit, token_network_deposit_limit
        )

    # failures with strings that are not addresses
    with pytest.raises(ValidationError):
        token_network_registry_contract.functions.createERC20TokenNetwork(
            "", channel_participant_deposit_limit, token_network_deposit_limit
        )
    with pytest.raises(ValidationError):
        token_network_registry_contract.functions.createERC20TokenNetwork(
            NOT_ADDRESS, channel_participant_deposit_limit, token_network_deposit_limit
        )

    # failures with addresses where no Token contract can be found
    with pytest.raises(TransactionFailed, match="TN: invalid token address"):
        token_network_registry_contract.functions.createERC20TokenNetwork(
            EMPTY_ADDRESS,
            channel_participant_deposit_limit,
            token_network_deposit_limit,
        ).call()
    with pytest.raises(TransactionFailed, match="TN: invalid token contract"):
        token_network_registry_contract.functions.createERC20TokenNetwork(
            A, channel_participant_deposit_limit, token_network_deposit_limit
        ).call()
    with pytest.raises(TransactionFailed):
        token_network_registry_contract.functions.createERC20TokenNetwork(
            fake_token_contract,
            channel_participant_deposit_limit,
            token_network_deposit_limit,
        ).call()

    # failures with invalid deposit limits
    with pytest.raises(TransactionFailed, match="TN: invalid participant limit"):
        token_network_registry_contract.functions.createERC20TokenNetwork(
            custom_token.address, 0, token_network_deposit_limit
        ).call({"from": DEPLOYER_ADDRESS})

    with pytest.raises(TransactionFailed, match="TN: invalid network deposit limit"):
        token_network_registry_contract.functions.createERC20TokenNetwork(
            custom_token.address, channel_participant_deposit_limit, 0
        ).call({"from": DEPLOYER_ADDRESS})

    with pytest.raises(TransactionFailed, match="TN: invalid deposit limits"):
        # fails because token_network_deposit_limit is smaller than
        # channel_participant_deposit_limit.
        token_network_registry_contract.functions.createERC20TokenNetwork(
            custom_token.address,
            token_network_deposit_limit,
            channel_participant_deposit_limit,
        ).call({"from": DEPLOYER_ADDRESS})

    # see a success to make sure above tests were meaningful
    call_and_transact(
        token_network_registry_contract.functions.createERC20TokenNetwork(
            custom_token.address,
            channel_participant_deposit_limit,
            token_network_deposit_limit,
        ),
        {"from": DEPLOYER_ADDRESS},
    )


@pytest.mark.usefixtures("no_token_network")
def test_create_erc20_token_network(
    register_token_network: Callable,
    token_network_registry_contract: Contract,
    custom_token: Contract,
    channel_participant_deposit_limit: int,
    token_network_deposit_limit: int,
) -> None:
    """Create a TokenNetwork through a TokenNetworkRegistry"""

    assert (
        token_network_registry_contract.functions.token_to_token_networks(
            custom_token.address
        ).call()
        == EMPTY_ADDRESS
    )

    token_network = register_token_network(
        token_network_registry=token_network_registry_contract,
        token_address=custom_token.address,
        channel_participant_deposit_limit=channel_participant_deposit_limit,
        token_network_deposit_limit=token_network_deposit_limit,
    )

    assert token_network.functions.token().call() == custom_token.address
    assert (
        token_network_registry_contract.functions.token_to_token_networks(
            custom_token.address
        ).call()
        == token_network.address
    )

    secret_registry = token_network_registry_contract.functions.secret_registry_address().call()
    assert token_network.functions.secret_registry().call() == secret_registry

    chain_id = token_network_registry_contract.functions.chain_id().call()
    assert token_network.functions.chain_id().call() == chain_id

    settle_timeout_min = token_network_registry_contract.functions.settlement_timeout_min().call()
    assert token_network.functions.settlement_timeout_min().call() == settle_timeout_min

    settle_timeout_max = token_network_registry_contract.functions.settlement_timeout_max().call()
    assert token_network.functions.settlement_timeout_max().call() == settle_timeout_max


@pytest.mark.usefixtures("no_token_network")
def test_create_erc20_token_network_twice_fails(
    token_network_registry_contract: Contract,
    custom_token: Contract,
    channel_participant_deposit_limit: int,
    token_network_deposit_limit: int,
) -> None:
    """Only one TokenNetwork should be creatable from a TokenNetworkRegistry"""

    call_and_transact(
        token_network_registry_contract.functions.createERC20TokenNetwork(
            custom_token.address,
            channel_participant_deposit_limit,
            token_network_deposit_limit,
        ),
        {"from": DEPLOYER_ADDRESS},
    )

    with pytest.raises(TransactionFailed, match="TNR: registry full"):
        token_network_registry_contract.functions.createERC20TokenNetwork(
            custom_token.address,
            channel_participant_deposit_limit,
            token_network_deposit_limit,
        ).call({"from": DEPLOYER_ADDRESS})


@pytest.mark.usefixtures("no_token_network")
def test_events(
    register_token_network: Callable,
    token_network_registry_contract: Contract,
    custom_token: Contract,
    event_handler: Callable,
    channel_participant_deposit_limit: int,
    token_network_deposit_limit: int,
) -> None:
    """TokenNetworkRegistry should raise an event when deploying a new TokenNetwork"""

    ev_handler = event_handler(token_network_registry_contract)

    new_token_network = register_token_network(
        token_network_registry=token_network_registry_contract,
        token_address=custom_token.address,
        channel_participant_deposit_limit=channel_participant_deposit_limit,
        token_network_deposit_limit=token_network_deposit_limit,
    )

    ev_handler.add(
        None,
        EVENT_TOKEN_NETWORK_CREATED,
        check_token_network_created(custom_token.address, new_token_network.address),
    )
    ev_handler.check()


@pytest.mark.usefixtures("no_token_network")
def test_change_owner(
    token_network_registry_contract: Contract,
    get_accounts: Callable,
) -> None:
    """Address must be allowed to remove limits after if became owner"""
    new_controller = get_accounts(1)[0]

    # Must fail when controller is still DEPLOYER_ADDRESS
    with pytest.raises(TransactionFailed, match="Can only be called by controller"):
        call_and_transact(
            token_network_registry_contract.functions.removeLimits(),
            {"from": new_controller},
        )

    # Must succeed after change of controller
    call_and_transact(
        token_network_registry_contract.functions.changeController(new_controller),
        {"from": DEPLOYER_ADDRESS},
    )
    call_and_transact(
        token_network_registry_contract.functions.removeLimits(),
        {"from": new_controller},
    )



from typing import Callable

import pytest
from eth_tester.exceptions import TransactionFailed
from web3 import Web3
from web3.contract import Contract

from raiden_contracts.constants import EMPTY_ADDRESS, MessageTypeId
from raiden_contracts.utils.proofs import eth_sign_hash_message, pack_balance_proof
from raiden_contracts.utils.signature import sign
from raiden_contracts.utils.type_aliases import ChainID, PrivateKey


# pylint: disable=E1120


def test_verify(
    web3: Web3,
    token_network: Contract,
    signature_test_contract: Contract,
    get_accounts: Callable,
    create_channel: Callable,
    create_balance_proof: Callable,
) -> None:
    """ECVerify.ecverify returns the correct address

    This test checks if the signature test contract returns the correct
    addresses on the balance hash signed by both ends of a channel"""
    (A, B) = get_accounts(2)
    channel_identifier = create_channel(A, B)[0]

    balance_proof_A = create_balance_proof(channel_identifier, A, 2, 0, 3)
    signature = balance_proof_A.original_signature
    balance_proof_hash = eth_sign_hash_message(
        pack_balance_proof(
            token_network_address=token_network.address,
            chain_identifier=ChainID(web3.eth.chain_id),
            channel_identifier=channel_identifier,
            balance_hash=balance_proof_A.balance_hash,
            nonce=balance_proof_A.nonce,
            additional_hash=balance_proof_A.additional_hash,
            msg_type=MessageTypeId.BALANCE_PROOF,
        )
    )
    address = signature_test_contract.functions.verify(balance_proof_hash, signature).call()
    assert address == A

    balance_proof_B = create_balance_proof(channel_identifier, B, 0, 0, 0)
    signature = balance_proof_B.original_signature
    balance_proof_hash = eth_sign_hash_message(
        pack_balance_proof(
            token_network_address=token_network.address,
            chain_identifier=ChainID(web3.eth.chain_id),
            channel_identifier=channel_identifier,
            balance_hash=balance_proof_B.balance_hash,
            nonce=balance_proof_B.nonce,
            additional_hash=balance_proof_B.additional_hash,
            msg_type=MessageTypeId.BALANCE_PROOF,
        )
    )
    address = signature_test_contract.functions.verify(balance_proof_hash, signature).call()
    assert address == B


def test_verify_fail(
    signature_test_contract: Contract, get_accounts: Callable, get_private_key: Callable
) -> None:
    """ECVerify.ecverify on failure cases

    the signature test contract returns the correct address on a correct
    message hash, returns a different address on a wrong message hash, and
    fails on a too long signature"""
    A = get_accounts(1)[0]
    message_hash = Web3.solidityKeccak(["string", "uint256"], ["hello", 5])
    signature = sign(get_private_key(A), message_hash, v=27)

    assert signature_test_contract.functions.verify(message_hash, signature).call() == A

    message_hash = Web3.solidityKeccak(["string", "uint256"], ["hello", 6])
    assert signature_test_contract.functions.verify(message_hash, signature).call() != A

    signature2 = signature[:65] + bytes([2])
    with pytest.raises(TransactionFailed):
        signature_test_contract.functions.verify(message_hash, signature2).call()


def test_ecrecover_output(
    web3: Web3,
    token_network: Contract,
    signature_test_contract: Contract,
    get_accounts: Callable,
    create_channel: Callable,
    create_balance_proof: Callable,
) -> None:
    """ecrecover returns the address that was used to sign a balance proof"""
    (A, B) = get_accounts(2)
    channel_identifier = create_channel(A, B)[0]
    balance_proof_A = create_balance_proof(channel_identifier, A, 2, 0, 3)
    signature = balance_proof_A.original_signature
    r = signature[:32]
    s = signature[32:64]
    v = signature[64:]
    balance_proof_hash = eth_sign_hash_message(
        pack_balance_proof(
            token_network_address=token_network.address,
            chain_identifier=ChainID(web3.eth.chain_id),
            channel_identifier=channel_identifier,
            balance_hash=balance_proof_A.balance_hash,
            nonce=balance_proof_A.nonce,
            additional_hash=balance_proof_A.additional_hash,
            msg_type=MessageTypeId.BALANCE_PROOF,
        )
    )

    address = signature_test_contract.functions.verifyEcrecoverOutput(
        balance_proof_hash, r, s, int.from_bytes(v, byteorder="big")
    ).call()
    assert address == A


def test_ecrecover_output_zero(
    signature_test_contract: Contract, get_accounts: Callable, get_private_key: Callable
) -> None:
    """ecrecover returns 0 for an incorrect value of the v parameter"""
    A = get_accounts(1)[0]
    privatekey = get_private_key(A)
    message_hash = Web3.solidityKeccak(["string", "uint256"], ["hello", 5])
    signature = sign(privatekey, message_hash, v=27)

    assert (
        signature_test_contract.functions.verifyEcrecoverOutput(
            message_hash, signature[:32], signature[32:64], 2
        ).call()
        == EMPTY_ADDRESS
    )


def test_ecrecover_output_fail(
    signature_test_contract: Contract, get_accounts: Callable, get_private_key: Callable
) -> None:
    """ecrecover detects a wrong message content and returns zero"""
    A = get_accounts(1)[0]
    privatekey = get_private_key(A)
    message_hash = Web3.solidityKeccak(["string", "uint256"], ["hello", 5])
    signature = sign(privatekey, message_hash, v=27)

    assert (
        signature_test_contract.functions.verifyEcrecoverOutput(
            message_hash,
            signature[:32],
            signature[32:64],
            int.from_bytes(signature[64:], byteorder="big"),
        ).call()
        == A
    )

    message_hash2 = Web3.solidityKeccak(["string", "uint256"], ["hello", 6])
    assert (
        signature_test_contract.functions.verifyEcrecoverOutput(
            message_hash2,
            signature[:32],
            signature[32:64],
            int.from_bytes(signature[64:], byteorder="big"),
        ).call()
        != A
    )


def test_sign_not_bytes(get_private_key: Callable, get_accounts: Callable) -> None:
    """sign() raises when message is not bytes"""
    A = get_accounts(1)[0]
    privatekey = get_private_key(A)
    with pytest.raises(TypeError):
        sign(privatekey, "a" * 32, v=27)  # type: ignore


def test_sign_not_32_bytes(get_private_key: Callable, get_accounts: Callable) -> None:
    """sign() raises when message is not exactly 32 bytes"""
    A = get_accounts(1)[0]
    privatekey = get_private_key(A)
    with pytest.raises(ValueError):
        sign(privatekey, bytes("a" * 31, "ascii"), v=27)


def test_sign_privatekey_not_string(get_private_key: Callable, get_accounts: Callable) -> None:
    """sign() raises when the private key is not a string"""
    A = get_accounts(1)[0]
    privatekey = get_private_key(A)
    with pytest.raises(TypeError):
        sign(PrivateKey(bytes(privatekey, "ascii")), bytes("a" * 32, "ascii"), v=27)


def test_sign_wrong_v(get_private_key: Callable, get_accounts: Callable) -> None:
    """sign() raises when the private key is not a string"""
    A = get_accounts(1)[0]
    privatekey = get_private_key(A)
    with pytest.raises(ValueError):
        sign(privatekey, bytes("a" * 32, "ascii"), v=22)



import os
import stat
import tempfile
from pathlib import Path

from raiden_contracts.utils.private_key import check_permission_safety, get_private_key


def test_permission_safety_different_uid() -> None:
    """check_permission_safety() should fail on a file with a different uid"""
    assert not check_permission_safety(Path("/"))


def test_permission_safety_group_writable() -> None:
    """check_permission_safety() should fail on a file that is writable to group"""
    with tempfile.NamedTemporaryFile() as tmpfile:
        orig = os.stat(tmpfile.name).st_mode
        os.chmod(tmpfile.name, stat.S_IWGRP | orig)
        assert not check_permission_safety(Path(tmpfile.name))


def test_permission_safety_executable() -> None:
    """check_permission_safety() should fail on a file that is executable to others"""
    with tempfile.NamedTemporaryFile() as tmpfile:
        orig = os.stat(tmpfile.name).st_mode
        os.chmod(tmpfile.name, stat.S_IXOTH | orig)
        assert not check_permission_safety(Path(tmpfile.name))


def test_get_private_key_empty_path() -> None:
    """get_private_key() should return None on a None key path"""
    assert get_private_key(None) is None  # type: ignore


def test_get_private_key_nonexistent() -> None:
    """get_private_key() should return None on a nonexistent file path"""
    assert get_private_key(Path("ggg")) is None


def test_get_private_key_writable_keyfile() -> None:
    """get_private_key() should return None on a key path with wrong permissions"""
    with tempfile.NamedTemporaryFile() as tmpfile:
        orig = os.stat(tmpfile.name).st_mode
        os.chmod(tmpfile.name, stat.S_IWGRP | orig)
        assert get_private_key(Path(tmpfile.name)) is None


def test_get_private_key_writable_password_file() -> None:
    """get_private_key() should return None on a password path with wrong permissions"""
    with tempfile.NamedTemporaryFile() as keyfile:
        with tempfile.NamedTemporaryFile() as password_file:
            orig = os.stat(password_file.name).st_mode
            os.chmod(password_file.name, stat.S_IWGRP | orig)
            assert (
                get_private_key(
                    key_path=Path(keyfile.name), password_path=Path(password_file.name)
                )
                is None
            )



import tempfile
from pathlib import Path

import pytest

from raiden_contracts.utils.file_ops import load_json_from_path


def test_load_json_from_corrupt_file() -> None:
    with tempfile.NamedTemporaryFile() as f:
        f.write(b"not a JSON")
        with pytest.raises(ValueError):
            load_json_from_path(Path(f.name))



from typing import Any, List, Optional

from coincurve import PrivateKey as CCPrivateKey
from eth_tester import EthereumTester
from eth_typing.evm import ChecksumAddress
from hexbytes import HexBytes
from web3 import Web3
from web3.contract import Contract, ContractFunction
from web3.providers.eth_tester import EthereumTesterProvider
from web3.types import TxParams

from raiden_contracts.constants import CONTRACT_CUSTOM_TOKEN, CONTRACT_TOKEN_NETWORK
from raiden_contracts.contract_manager import ContractManager
from raiden_contracts.tests.utils.constants import FAUCET_ALLOWANCE
from raiden_contracts.utils.signature import private_key_to_address


def get_web3(eth_tester: EthereumTester, deployer_key: CCPrivateKey) -> Web3:
    """Returns an initialized Web3 instance"""
    provider = EthereumTesterProvider(eth_tester)
    web3 = Web3(provider)

    # add faucet account to tester
    eth_tester.add_account(deployer_key.to_hex())

    # make faucet rich
    eth_tester.send_transaction(
        {
            "from": eth_tester.get_accounts()[0],
            "to": private_key_to_address(deployer_key.to_hex()),
            "gas": 21000,
            "value": FAUCET_ALLOWANCE,
        }
    )

    return web3


def deploy_contract(
    web3: Web3,
    contracts_manager: ContractManager,
    contract_name: str,
    deployer_key: CCPrivateKey,
    args: List[Any],
) -> Contract:
    deployer_address = private_key_to_address(deployer_key.to_hex())
    json_contract = contracts_manager.get_contract(contract_name)
    contract = web3.eth.contract(abi=json_contract["abi"], bytecode=json_contract["bin"])
    tx_hash = contract.constructor(*args).transact(TxParams({"from": deployer_address}))
    contract_address = web3.eth.get_transaction_receipt(tx_hash)["contractAddress"]

    return contract(contract_address)


def deploy_custom_token(
    web3: Web3, deployer_key: CCPrivateKey, contract_manager: ContractManager
) -> Contract:
    return deploy_contract(
        web3=web3,
        contracts_manager=contract_manager,
        contract_name=CONTRACT_CUSTOM_TOKEN,
        deployer_key=deployer_key,
        args=[10 ** 26, 18, CONTRACT_CUSTOM_TOKEN, "TKN"],
    )


def get_token_network(
    web3: Web3, address: ChecksumAddress, contracts_manager: ContractManager
) -> Contract:
    json_contract = contracts_manager.get_contract(CONTRACT_TOKEN_NETWORK)

    return web3.eth.contract(abi=json_contract["abi"], address=address)


def call_and_transact(
    contract_function: ContractFunction,
    transaction_params: Optional[TxParams] = None,
) -> HexBytes:
    """Executes contract_function.{call, transaction}(transaction_params) and returns txhash"""
    # First 'call' might raise an exception
    contract_function.call(transaction_params)
    return contract_function.transact(transaction_params)



from typing import Callable

import pytest
from web3.contract import Contract

from raiden_contracts.constants import CONTRACT_MONITORING_SERVICE


@pytest.fixture(scope="session")
def monitoring_service_external(
    deploy_tester_contract: Callable,
    custom_token: Contract,
    service_registry: Contract,
    uninitialized_user_deposit_contract: Contract,
    token_network_registry_contract: Contract,
) -> Contract:
    return deploy_tester_contract(
        CONTRACT_MONITORING_SERVICE,
        _token_address=custom_token.address,
        _service_registry_address=service_registry.address,
        _udc_address=uninitialized_user_deposit_contract.address,
        _token_network_registry_address=token_network_registry_contract.address,
    )


@pytest.fixture()
def monitoring_service_internals(
    custom_token: Contract,
    service_registry: Contract,
    uninitialized_user_deposit_contract: Contract,
    deploy_tester_contract: Callable,
    token_network_registry_contract: Contract,
) -> Contract:
    return deploy_tester_contract(
        "MonitoringServiceInternalsTest",
        _token_address=custom_token.address,
        _service_registry_address=service_registry.address,
        _udc_address=uninitialized_user_deposit_contract.address,
        _token_network_registry_address=token_network_registry_contract.address,
    )



from enum import Enum
from typing import Optional

from semantic_version import SimpleSpec, Version


class ContractFeature(Enum):
    SERVICES = "services"
    MAX_TOKEN_NETWORKS = "max_token_networks"
    INITIAL_SERVICE_DEPOSIT = "initial_service_deposit"
    MS_NEEDS_TOKENNETWORK_REGISTRY = "ms_needs_tokennetwork_registry"


CONTRACT_FEATURE_VERSIONS = {
    ContractFeature.SERVICES: SimpleSpec(">=0.8.0"),
    ContractFeature.MAX_TOKEN_NETWORKS: SimpleSpec(">=0.9.0"),
    ContractFeature.INITIAL_SERVICE_DEPOSIT: SimpleSpec(">=0.18.0"),
    ContractFeature.MS_NEEDS_TOKENNETWORK_REGISTRY: SimpleSpec(">=0.22.0"),
}


def _matches_feature(feature: ContractFeature, version: Optional[str]) -> bool:
    """Returns a bool indicating whether the passed version matches the minimum required
    version for the given feature."""

    if version is None:
        # contracts_version == None means the stock version in development.
        return True
    return CONTRACT_FEATURE_VERSIONS[feature].match(Version(version))


def contracts_version_with_max_token_networks(version: Optional[str]) -> bool:
    return _matches_feature(ContractFeature.MAX_TOKEN_NETWORKS, version)


def contracts_version_provides_services(version: Optional[str]) -> bool:
    return _matches_feature(ContractFeature.SERVICES, version)


def contracts_version_has_initial_service_deposit(version: Optional[str]) -> bool:
    return _matches_feature(ContractFeature.INITIAL_SERVICE_DEPOSIT, version)


def contracts_version_monitoring_service_takes_token_network_registry(
    version: Optional[str],
) -> bool:
    return _matches_feature(ContractFeature.MS_NEEDS_TOKENNETWORK_REGISTRY, version)
